@@ -1,5 +1,5 @@
 /* Clzip - LZMA lossless data compressor
-   Copyright (C) 2010-2021 Antonio Diaz Diaz.
+   Copyright (C) 2010-2022 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -188,10 +188,9 @@ static inline int LZeb_price_rep( const struct LZ_encoder_base * const eb,
                                   const int rep, const State state,
                                   const int pos_state )
   {
-  int price;
   if( rep == 0 ) return price0( eb->bm_rep0[state] ) +
                         price1( eb->bm_len[state][pos_state] );
-  price = price1( eb->bm_rep0[state] );
+  int price = price1( eb->bm_rep0[state] );
   if( rep == 1 )
     price += price0( eb->bm_rep1[state] );
   else
