@@ -1,5 +1,5 @@
 /* Clzip - LZMA lossless data compressor
-   Copyright (C) 2010-2021 Antonio Diaz Diaz.
+   Copyright (C) 2010-2022 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
    Exit status: 0 for a normal exit, 1 for environmental problems
    (file not found, invalid flags, I/O errors, etc), 2 to indicate a
    corrupt or invalid input file, 3 for an internal consistency error
-   (eg, bug) which caused clzip to panic.
+   (e.g., bug) which caused clzip to panic.
 */
 
 #define _FILE_OFFSET_BITS 64
@@ -36,9 +36,9 @@
 #include <unistd.h>
 #include <utime.h>
 #include <sys/stat.h>
-#if defined(__MSVCRT__) || defined(__OS2__) || defined(__DJGPP__)
+#if defined __MSVCRT__ || defined __OS2__ || defined __DJGPP__
 #include <io.h>
-#if defined(__MSVCRT__)
+#if defined __MSVCRT__
 #define fchmod(x,y) 0
 #define fchown(x,y,z) 0
 #define strtoull strtoul
@@ -51,7 +51,7 @@
 #define S_IWOTH 0
 #endif
 #endif
-#if defined(__DJGPP__)
+#if defined __DJGPP__
 #define S_ISSOCK(x) 0
 #define S_ISVTX 0
 #endif
@@ -72,10 +72,15 @@
 #error "Environments where CHAR_BIT != 8 are not supported."
 #endif
 
+#if ( defined  SIZE_MAX &&  SIZE_MAX < UINT_MAX ) || \
+    ( defined SSIZE_MAX && SSIZE_MAX <  INT_MAX )
+#error "Environments where 'size_t' is narrower than 'int' are not supported."
+#endif
+
 int verbosity = 0;
 
 static const char * const program_name = "clzip";
-static const char * const program_year = "2021";
+static const char * const program_year = "2022";
 static const char * invocation_name = "clzip";		/* default value */
 
 static const struct { const char * from; const char * to; } known_extensions[] = {
@@ -106,13 +111,14 @@ static void show_help( void )
           "C++ compiler.\n"
           "\nLzip is a lossless data compressor with a user interface similar to the one\n"
           "of gzip or bzip2. Lzip uses a simplified form of the 'Lempel-Ziv-Markov\n"
-          "chain-Algorithm' (LZMA) stream format, chosen to maximize safety and\n"
-          "interoperability. Lzip can compress about as fast as gzip (lzip -0) or\n"
-          "compress most files more than bzip2 (lzip -9). Decompression speed is\n"
-          "intermediate between gzip and bzip2. Lzip is better than gzip and bzip2 from\n"
-          "a data recovery perspective. Lzip has been designed, written, and tested\n"
-          "with great care to replace gzip and bzip2 as the standard general-purpose\n"
-          "compressed format for unix-like systems.\n"
+          "chain-Algorithm' (LZMA) stream format and provides a 3 factor integrity\n"
+          "checking to maximize interoperability and optimize safety. Lzip can compress\n"
+          "about as fast as gzip (lzip -0) or compress most files more than bzip2\n"
+          "(lzip -9). Decompression speed is intermediate between gzip and bzip2.\n"
+          "Lzip is better than gzip and bzip2 from a data recovery perspective. Lzip\n"
+          "has been designed, written, and tested with great care to replace gzip and\n"
+          "bzip2 as the standard general-purpose compressed format for unix-like\n"
+          "systems.\n"
           "\nUsage: %s [options] [files]\n", invocation_name );
   printf( "\nOptions:\n"
           "  -h, --help                     display this help and exit\n"
@@ -150,7 +156,7 @@ static void show_help( void )
           "'tar -xf foo.tar.lz' or 'clzip -cd foo.tar.lz | tar -xf -'.\n"
           "\nExit status: 0 for a normal exit, 1 for environmental problems (file\n"
           "not found, invalid flags, I/O errors, etc), 2 to indicate a corrupt or\n"
-          "invalid input file, 3 for an internal consistency error (eg, bug) which\n"
+          "invalid input file, 3 for an internal consistency error (e.g., bug) which\n"
           "caused clzip to panic.\n"
           "\nThe ideas embodied in clzip are due to (at least) the following people:\n"
           "Abraham Lempel and Jacob Ziv (for the LZ algorithm), Andrey Markov (for the\n"
@@ -194,8 +200,6 @@ struct Pretty_print
 static void Pp_init( struct Pretty_print * const pp,
                      const char * const filenames[], const int num_filenames )
   {
-  unsigned stdin_name_len;
-  int i;
   pp->name = 0;
   pp->padded_name = 0;
   pp->stdin_name = "(stdin)";
@@ -203,7 +207,8 @@ static void Pp_init( struct Pretty_print * const pp,
   pp->first_post = false;
 
   if( verbosity <= 0 ) return;
-  stdin_name_len = strlen( pp->stdin_name );
+  const unsigned stdin_name_len = strlen( pp->stdin_name );
+  int i;
   for( i = 0; i < num_filenames; ++i )
     {
     const char * const s = filenames[i];
@@ -237,16 +242,14 @@ static void Pp_reset( struct Pretty_print * const pp )
 
 void Pp_show_msg( struct Pretty_print * const pp, const char * const msg )
   {
-  if( verbosity >= 0 )
+  if( verbosity < 0 ) return;
+  if( pp->first_post )
     {
-    if( pp->first_post )
-      {
-      pp->first_post = false;
-      fputs( pp->padded_name, stderr );
-      if( !msg ) fflush( stderr );
-      }
-    if( msg ) fprintf( stderr, "%s\n", msg );
+    pp->first_post = false;
+    fputs( pp->padded_name, stderr );
+    if( !msg ) fflush( stderr );
     }
+  if( msg ) fprintf( stderr, "%s\n", msg );
   }
 
 
@@ -284,17 +287,53 @@ void show_header( const unsigned dictionary_size )
   }
 
 
-static unsigned long long getnum( const char * const ptr,
+/* separate large numbers >= 100_000 in groups of 3 digits using '_' */
+static const char * format_num3( unsigned long long num )
+  {
+  const char * const si_prefix = "kMGTPEZY";
+  const char * const binary_prefix = "KMGTPEZY";
+  enum { buffers = 8, bufsize = 4 * sizeof (long long) };
+  static char buffer[buffers][bufsize];	/* circle of static buffers for printf */
+  static int current = 0;
+  int i;
+  char * const buf = buffer[current++]; current %= buffers;
+  char * p = buf + bufsize - 1;		/* fill the buffer backwards */
+  *p = 0;	/* terminator */
+  if( num > 1024 )
+    {
+    char prefix = 0;			/* try binary first, then si */
+    for( i = 0; i < 8 && num >= 1024 && num % 1024 == 0; ++i )
+      { num /= 1024; prefix = binary_prefix[i]; }
+    if( prefix ) *(--p) = 'i';
+    else
+      for( i = 0; i < 8 && num >= 1000 && num % 1000 == 0; ++i )
+        { num /= 1000; prefix = si_prefix[i]; }
+    if( prefix ) *(--p) = prefix;
+    }
+  const bool split = num >= 100000;
+
+  for( i = 0; ; )
+    {
+    *(--p) = num % 10 + '0'; num /= 10; if( num == 0 ) break;
+    if( split && ++i >= 3 ) { i = 0; *(--p) = '_'; }
+    }
+  return p;
+  }
+
+
+static unsigned long long getnum( const char * const arg,
+                                  const char * const option_name,
                                   const unsigned long long llimit,
                                   const unsigned long long ulimit )
   {
-  unsigned long long result;
   char * tail;
   errno = 0;
-  result = strtoull( ptr, &tail, 0 );
-  if( tail == ptr )
+  unsigned long long result = strtoull( arg, &tail, 0 );
+  if( tail == arg )
     {
-    show_error( "Bad or missing numerical argument.", 0, true );
+    if( verbosity >= 0 )
+      fprintf( stderr, "%s: Bad or missing numerical argument in "
+               "option '%s'.\n", program_name, option_name );
     exit( 1 );
     }
 
@@ -317,7 +356,9 @@ static unsigned long long getnum( const char * const ptr,
       }
     if( exponent <= 0 )
       {
-      show_error( "Bad multiplier in numerical argument.", 0, true );
+      if( verbosity >= 0 )
+        fprintf( stderr, "%s: Bad multiplier in numerical argument of "
+                 "option '%s'.\n", program_name, option_name );
       exit( 1 );
       }
     for( i = 0; i < exponent; ++i )
@@ -329,21 +370,24 @@ static unsigned long long getnum( const char * const ptr,
   if( !errno && ( result < llimit || result > ulimit ) ) errno = ERANGE;
   if( errno )
     {
-    show_error( "Numerical argument out of limits.", 0, false );
+    if( verbosity >= 0 )
+      fprintf( stderr, "%s: Numerical argument out of limits [%s,%s] "
+               "in option '%s'.\n", program_name, format_num3( llimit ),
+               format_num3( ulimit ), option_name );
     exit( 1 );
     }
   return result;
   }
 
 
-static int get_dict_size( const char * const arg )
+static int get_dict_size( const char * const arg, const char * const option_name )
   {
   char * tail;
   const long bits = strtol( arg, &tail, 0 );
   if( bits >= min_dictionary_bits &&
       bits <= max_dictionary_bits && *tail == 0 )
     return 1 << bits;
-  return getnum( arg, min_dictionary_size, max_dictionary_size );
+  return getnum( arg, option_name, min_dictionary_size, max_dictionary_size );
   }
 
 
@@ -519,7 +563,7 @@ static bool check_tty_in( const char * const input_filename, const int infd,
       isatty( infd ) )				/* for example /dev/tty */
     { show_file_error( input_filename,
                        "I won't read compressed data from a terminal.", 0 );
-      close( infd ); set_retval( retval, 1 );
+      close( infd ); set_retval( retval, 2 );
       if( program_mode != m_test ) cleanup_and_fail( *retval );
       return false; }
   return true;
@@ -600,7 +644,6 @@ static int compress( const unsigned long long cfile_size,
                      struct Pretty_print * const pp,
                      const struct stat * const in_statsp, const bool zero )
   {
-  unsigned long long in_size = 0, out_size = 0, partial_volume_size = 0;
   int retval = 0;
   struct Poly_encoder encoder = { 0, 0, 0 };	/* polymorphic encoder */
   if( verbosity >= 1 ) Pp_show_msg( pp, 0 );
@@ -633,6 +676,7 @@ static int compress( const unsigned long long cfile_size,
     }
   }
 
+  unsigned long long in_size = 0, out_size = 0, partial_volume_size = 0;
   while( true )			/* encode one member per iteration */
     {
     const unsigned long long size = ( volume_size > 0 ) ?
@@ -729,12 +773,9 @@ static int decompress( const unsigned long long cfile_size, const int infd,
 
   for( first_member = true; ; first_member = false )
     {
-    int result, size;
-    unsigned dictionary_size;
     Lzip_header header;
-    struct LZ_decoder decoder;
     Rd_reset_member_position( &rdec );
-    size = Rd_read_data( &rdec, header, Lh_size );
+    const int size = Rd_read_data( &rdec, header, Lh_size );
     if( Rd_finished( &rdec ) )			/* End Of File */
       {
       if( first_member )
@@ -764,17 +805,18 @@ static int decompress( const unsigned long long cfile_size, const int infd,
     if( !Lh_verify_version( header ) )
       { Pp_show_msg( pp, bad_version( Lh_version( header ) ) );
         retval = 2; break; }
-    dictionary_size = Lh_get_dictionary_size( header );
+    const unsigned dictionary_size = Lh_get_dictionary_size( header );
     if( !isvalid_ds( dictionary_size ) )
       { Pp_show_msg( pp, bad_dict_msg ); retval = 2; break; }
 
     if( verbosity >= 2 || ( verbosity == 1 && first_member ) )
       Pp_show_msg( pp, 0 );
 
+    struct LZ_decoder decoder;
     if( !LZd_init( &decoder, &rdec, dictionary_size, outfd ) )
       { Pp_show_msg( pp, mem_msg ); retval = 1; break; }
     show_dprogress( cfile_size, partial_file_pos, &rdec, pp );	/* init */
-    result = LZd_decode_member( &decoder, pp );
+    const int result = LZd_decode_member( &decoder, pp );
     partial_file_pos += Rd_member_position( &rdec );
     LZd_free( &decoder );
     if( result != 0 )
@@ -911,24 +953,16 @@ int main( const int argc, const char * const argv[] )
   unsigned long long member_size = max_member_size;
   unsigned long long volume_size = 0;
   const char * default_output_filename = "";
-  static struct Arg_parser parser;	/* static because valgrind complains */
-  static struct Pretty_print pp;	/* and memory management in C sucks */
-  static const char ** filenames = 0;
-  int num_filenames = 0;
   enum Mode program_mode = m_compress;
-  int argind = 0;
-  int failed_tests = 0;
-  int retval = 0;
   int i;
-  bool filenames_given = false;
   bool force = false;
   bool ignore_trailing = true;
   bool keep_input_files = false;
   bool loose_trailing = false;
   bool recompress = false;
-  bool stdin_used = false;
   bool to_stdout = false;
   bool zero = false;
+  if( argc > 0 ) invocation_name = argv[0];
 
   enum { opt_lt = 256 };
   const struct ap_Option options[] =
@@ -964,19 +998,22 @@ int main( const int argc, const char * const argv[] )
     { opt_lt, "loose-trailing", ap_no  },
     {  0, 0,                    ap_no  } };
 
-  if( argc > 0 ) invocation_name = argv[0];
   CRC32_init();
 
+  /* static because valgrind complains and memory management in C sucks */
+  static struct Arg_parser parser;
   if( !ap_init( &parser, argc, argv, options, 0 ) )
     { show_error( mem_msg, 0, false ); return 1; }
   if( ap_error( &parser ) )				/* bad option */
     { show_error( ap_error( &parser ), 0, true ); return 1; }
 
+  int argind = 0;
   for( ; argind < ap_arguments( &parser ); ++argind )
     {
     const int code = ap_code( &parser, argind );
-    const char * const arg = ap_argument( &parser, argind );
     if( !code ) break;					/* no more options */
+    const char * const pn = ap_parsed_name( &parser, argind );
+    const char * const arg = ap_argument( &parser, argind );
     switch( code )
       {
       case '0': case '1': case '2': case '3': case '4':
@@ -984,7 +1021,7 @@ int main( const int argc, const char * const argv[] )
                 zero = ( code == '0' );
                 encoder_options = option_mapping[code-'0']; break;
       case 'a': ignore_trailing = false; break;
-      case 'b': member_size = getnum( arg, 100000, max_member_size ); break;
+      case 'b': member_size = getnum( arg, pn, 100000, max_member_size ); break;
       case 'c': to_stdout = true; break;
       case 'd': set_mode( &program_mode, m_decompress ); break;
       case 'f': force = true; break;
@@ -993,15 +1030,15 @@ int main( const int argc, const char * const argv[] )
       case 'k': keep_input_files = true; break;
       case 'l': set_mode( &program_mode, m_list ); break;
       case 'm': encoder_options.match_len_limit =
-                  getnum( arg, min_match_len_limit, max_match_len );
+                  getnum( arg, pn, min_match_len_limit, max_match_len );
                 zero = false; break;
       case 'n': break;
       case 'o': if( strcmp( arg, "-" ) == 0 ) to_stdout = true;
                 else { default_output_filename = arg; } break;
       case 'q': verbosity = -1; break;
-      case 's': encoder_options.dictionary_size = get_dict_size( arg );
+      case 's': encoder_options.dictionary_size = get_dict_size( arg, pn );
                 zero = false; break;
-      case 'S': volume_size = getnum( arg, 100000, max_volume_size ); break;
+      case 'S': volume_size = getnum( arg, pn, 100000, max_volume_size ); break;
       case 't': set_mode( &program_mode, m_test ); break;
       case 'v': if( verbosity < 4 ) ++verbosity; break;
       case 'V': show_version(); return 0;
@@ -1010,15 +1047,17 @@ int main( const int argc, const char * const argv[] )
       }
     } /* end process options */
 
-#if defined(__MSVCRT__) || defined(__OS2__) || defined(__DJGPP__)
+#if defined __MSVCRT__ || defined __OS2__ || defined __DJGPP__
   setmode( STDIN_FILENO, O_BINARY );
   setmode( STDOUT_FILENO, O_BINARY );
 #endif
 
-  num_filenames = max( 1, ap_arguments( &parser ) - argind );
+  static const char ** filenames = 0;
+  int num_filenames = max( 1, ap_arguments( &parser ) - argind );
   filenames = resize_buffer( filenames, num_filenames * sizeof filenames[0] );
   filenames[0] = "-";
 
+  bool filenames_given = false;
   for( i = 0; argind + i < ap_arguments( &parser ); ++i )
     {
     filenames[i] = ap_argument( &parser, argind + i );
@@ -1052,17 +1091,18 @@ int main( const int argc, const char * const argv[] )
   if( !to_stdout && program_mode != m_test && ( filenames_given || to_file ) )
     set_signals( signal_handler );
 
+  static struct Pretty_print pp;
   Pp_init( &pp, filenames, num_filenames );
 
+  int failed_tests = 0;
+  int retval = 0;
   const bool one_to_one = !to_stdout && program_mode != m_test && !to_file;
+  bool stdin_used = false;
   for( i = 0; i < num_filenames; ++i )
     {
-    unsigned long long cfile_size;
     const char * input_filename = "";
     int infd;
-    int tmp;
     struct stat in_stats;
-    const struct stat * in_statsp;
 
     Pp_set_name( &pp, filenames[i] );
     if( strcmp( filenames[i], "-" ) == 0 )
@@ -1104,9 +1144,12 @@ int main( const int argc, const char * const argv[] )
         return 1;	/* check tty only once and don't try to delete a tty */
       }
 
-    in_statsp = ( input_filename[0] && one_to_one ) ? &in_stats : 0;
-    cfile_size = ( input_filename[0] && S_ISREG( in_stats.st_mode ) ) ?
-      ( in_stats.st_size + 99 ) / 100 : 0;
+    const struct stat * const in_statsp =
+      ( input_filename[0] && one_to_one ) ? &in_stats : 0;
+    const unsigned long long cfile_size =
+      ( input_filename[0] && S_ISREG( in_stats.st_mode ) ) ?
+        ( in_stats.st_size + 99 ) / 100 : 0;
+    int tmp;
     if( program_mode == m_compress )
       tmp = compress( cfile_size, member_size, volume_size, infd,
                       &encoder_options, &pp, in_statsp, zero );
